#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <signal.h>
#include <unistd.h>

#include "sed_sig.h"
#include "mikmod.h"

curmod cur_mod;

int quiet=0;           /* set if quiet mode is enabled */
UBYTE md_type=0;       /* default is a non-wavetable sound device */

void tickhandler(void)
{
	MP_HandleTick();    /* play 1 tick of the module */
	MD_SetBPM(mp_bpm);
}

void fin_un(int x)
{
  MP_SetPosition(9);
}

void fin_deux(int x)
{
  MP_SetPosition(0);
}

void main_sound(char *nom, int options, int pere)
{
	extern int quit;
	extern int play_current;
	UNIMOD *mf;
	int cmderr=0;                   /* error in commandline flag */
	int morehelp=0;                 /* set if user wants more help */
	int t;
	extern float speed_constant;   /* tempo multiplier, initialised to 1*/
	UWORD reppos;

	int tolerant    =0;
	play_current    =1;	/* we are playing _this_ mod; jump to next 
				   when this goes to 0 */
	md_mixfreq      =44100;                     /* standard mixing freq */
	md_dmabufsize   =32768;                     /* standard dma buf size */
	md_mode         =DMODE_16BITS|DMODE_STEREO|DMODE_INTERP; /* standard mixing mode */
	md_device       =0;                  /* standard device: autodetect */

	signal(SIG_FIN_SON_1, fin_un);
	signal(SIG_FIN_SON_2, fin_deux);

	ML_RegisterLoader(&load_xm);
        MD_RegisterDriver(&drv_vox);
	MD_RegisterPlayer(tickhandler);
	/* Parse option switches using standard getopt function: */

#if 0
	opterr=0;
	while( !cmderr &&
#ifdef CPUTIME_SNAGGER
		(t=getopt(nargc,nargv,"ohxpm8irv:f:l:d:tsS")) != EOF ){
#else
		(t=getopt(nargc,nargv,"ohxpm8irv:f:l:d:t")) != EOF ){
#endif

		switch(t){

			case 'd':
				md_device=atoi(optarg);
				break;

			case 'l':
				if(optarg[0]=='d') MD_InfoDriver();
				else if(optarg[0]=='l') ML_InfoLoader();
				else{
					cmderr=1;
					break;
				}
				exit(0);

			case 'r':
				mp_loop=1;
				break;

			case 'm':
				md_mode&=~DMODE_STEREO;
				break;

			case '8':
				md_mode&=~DMODE_16BITS;
				break;

			case 'i':
				md_mode|=DMODE_INTERP;
				break;

			case 'x':
				mp_extspd=0;
				break;

			case 'p':
				mp_panning=0;
				break;

			case 'v':
				if((mp_volume=atoi(optarg))>100) mp_volume=100;
				break;

			case 'f':
				md_mixfreq=atol(optarg);
				break;

			case 'h':
				morehelp=1;
				cmderr=1;
				break;

			case 't':
				tolerant=1;
				break;

			case 'q':
				quiet=1;
				break;

#ifdef CPUTIME_SNAGGER
			case 's':
				if (nice(-20) == -1)
					perror("renice to -20");
				break;
			case 'S':
				{
					struct sched_param sp;
					memset(&sp, 0, sizeof(struct sched_param));
					sp.sched_priority = 1;
					if (sched_setscheduler(0, SCHED_RR, &sp) == -1)
						perror("realtime priority");
				}
				break;
#endif /* CPUTIME_SNAGGER */

			case '?':
				puts("\07Invalid switch or option needs an argument\n");
				cmderr=1;
				break;
		}
	}

	if(cmderr || optind>=nargc){

		/*
			there was an error in the commandline, or there were no true
			arguments, so display a usage message
		*/

                printf("Usage: %s [switches] <fletch.mod> ... \n",nargv[0]);

                printf("Type %s -h for more help.\n",nargv[0]);

                exit(-1);
	}
#endif

	/*  initialize soundcard */

	if(!MD_Init()){
		printf("Driver error: %s.\n",myerr);
		kill(pere, SIG_PASOK);
		exit(1);
	}

	/*  initialize volume and tempo multipliers */

	speed_constant=1.0;
	mp_volume=100;

	MD_PatternChange();

	/* load the module */

	mf=ML_LoadFN(nom);

	/* didn't work -> exit with errormsg. */

	if(mf==NULL){
		printf("MikMod Error: %s\n",myerr);
		MD_Exit();
		kill(pere, SIG_PASOK);
		exit(1);
	}

	kill(pere, SIG_OK);

	MP_Init(mf);
	md_numchn=mf->numchn;

	MP_SetPosition(12);

	MD_PlayStart();

	while(!MP_Ready())
		MD_Update();

	MD_PlayStop();          /* stop playing */
	ML_Free(mf);            /* and free the module */
	MD_Exit();
}
